\ init.fs -- I/O initialization for ITEST-850 -- 19Aug13 rjn
\
]  here  \ start marker
\ 
0 [if] ---------------------------[ Notes ]------------------------------------

1. Configures 850 chip on toolstick board.
2. To be called as part of cold so that after a reset the hardware 
    returns to these settings
3. If a bit needs changing in the application, "and/orl" it.
4. 850 toolstick I/O assignments (QSOP 24):

   Port	Pin Type Function
   ----	--- ---- --------------------------------------------------------------
   P0.0	  4   BD C2CK drive to Target, RST voltage from Target (chip detect)
   P0.1	  3   BD C2D drive to/from Target (bi-directional)
   P0.2    2   PP RS to Target (not needed)
   P0.3	 23   PP unassigned
   P0.4	 22   PP SERIAL TX
   P0.5	 21   DI SERIAL RX
   P0.6   20   PP unassigned
   P0.7	 19   PP C2D (to prog programming adapter)
   P1.0	 18   PP unassigned
   P1.1	 17   PP toolstick LED DS2
   P1.2   16   AI toolstick pot
   P1.3	 15   PP unassigned
   P1.4	 14   PP unassigned
   P1.5	 11   PP unassigned
   P1.6   10   PP unassigned
   P1.7    9   DI toolstick switch (s1)
   P2.0    8   BD C2D
   P2.1   12   DI toolstick switch (s2)
           5   -- GND
           6   -- VDD (3.3 Volts)
           7   -- RST/C2CK (to prog debug adapter)
     1,13,24   -- N/C  
\ 
[then] \ ----------------------------------------------------------------------
\ 
\ -----[ I/O ]
\ 
\ these are used by c2.fs
\ 
:m c2ck    [ 0 .P0 ] m;
:m c2d     [ 1 .P0 ] m;
:m rs      [ 2 .P0 ] m;  \ not needed -- c2 can reset

:m p10     [ 0 .P1 ] m;  \ toolstick LED (turned off at reset)
:m p11     [ 1 .P1 ] m;  \ toolstick LED (turned on at reset)
:m s17     [ 7 .P1 ] m;  \ first toolstick switch

:m tc2d    [ 0 .P2 ] m;  \ Target c2d line (connects to programmer)
:m s21     [ 1 .P2 ] m;  \ second toolstick switch
\ 
-: c2d-in  $fd # P0MDOUT and!  [ c2d set ] ;
\ -: c2d-out $df # P0MDOUT ior!  ;
-: c2d-out $02 # P0MDOUT ior!  ;
\ 
-: c2ck-in  $fe # P0MDOUT and!  [ c2ck set ] ;
-: c2ck-out $01 # P0MDOUT ior!  ;
\ 
-: tc2d-in  $fe # P2MDOUT and!  [ tc2d set ] ;
-: tc2d-out $01 # P2MDOUT ior!  ;

: t11  [ p11 toggle ] ;  \ twiddles P1.1
: t10  [ p10 toggle ] ;  \ twiddles P1.0  \ causes 'slow" problem
\ 
0 [if] \ -------------------[ I/O initialization ]-----------------------------

 1. Steps for port I/O initialization, per datasheet:
	Step 1 - Select input mode (analog or digital) for all port pins
		 using PnMDIN.  If the pin is in analog mode, a "1" must also
		 be written to the corresponding Port Latch.  Default for
		 PnMDIN is $ff (pins not configured as analog inputs).
	Step 2 - Select the output mode using PnMDOUT.  
		  Output Modes - 0=open drain, 1=push pull
	Step 3 - Assign port pins to desired peripherals using XBRn registers.
	Step 4 - Enable the crossbar (XBARE="1")
2. Pins used as comparator or ADC inputs should be configured as analog inputs.
3. Analog input pins should be skipped by the crossbar.
4. All analog pins must have a "1" set in the corresponding Port Latch reg.
[then] \ ----------------------------------------------------------------------
\ 
\ 
-: /chip 
   $00 # XBR2 #!  \ disable xbr
\ 
\ -----[ configure I/O ] 
\ 
   $dd # P0MDOUT #!     \ P0[0,2,3, 4, 6,7] push/pull
   $7f # P1MDOUT #!     \ push/pull P1[0-6] -- ts switch 1 on P1.7, LED on P1.1) 
   $01 # P2MDOUT #!     \ push/pull P2 (bit 0) -- ts switch 2 on P2.1
   $00 # CLKSEL #!      \ 24.5 MHz internal clock
\   $07 # PRTDRV #!      \ enable all as high drive (default)
   $01 # XBR0 #!        \ UART enabled
   $00 # XBR1 #!        \ no resources routed to port pins
   $c0 # XBR2 #!        \ enable XBR, weak pullups disabled
\ 
\ -----[ init application I/O ]
\ 
   [ c2ck-in  c2d-in  tc2d-in  s17 set  s21 set  p11 clr ] ;
\ 
\ 
here [ swap -  ( -- n)
cs? [if] cr .( init.fs             ) . .( bytes) [else] drop [then] 
]

\
\
0 [if] ---------------------[ Revision History ]-------------------------------

Date	  By  Description
======= === ===================================================================
17Aug13 rjn changed for 850 chip
10aug13 rjn changed for conditional compilation summary
09Feb13 rjn cloned from init-dcon.fs

[then] \ ----------------------------------------------------------------------

